/*
 * LADD Roller Derby Scoreboard
 * Copyright © 2011  Neale Pickett <neale@woozle.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* You can only have one scoreboard per page.  This limitation is mostly
 * because elements need specific id= attributes, and these attribumets
 * must be unique within a page.
 *
 * Corollary: don't change element ids without making a corresponding
 * change in this file.
 */

longnames = false;
tenths = true;

/* State names */
var SETUP = 0;                  // !P 30:00   !J 2:00
var JAM = 1;                    //  P          J 2:00
var LINEUP = 2;                 //  P          J 1:00
var TIMEOUT = 3;                // !P          J 1:00

var periodtext = ["Period 1", "Halftime", "Period 2", "Break"];
var jamtext = ["Jam", "Lineup", "Timeout", "Setup"];
var period = 0;
var jamno = 0;

var state = SETUP;

var timer_updates = [];
function update() {
    for (i in timer_updates) {
        var u = timer_updates[i];

        u();
    }
}

// Create a timer on [element].
// If [tenths] is true, show tenths of a second.
// If [callback] is defined, call it when time runs out.
function startTimer(element, tenths, callback) {
    var startTime;
    var running = false;
    var set_duration = 0;
    var duration = 0;
    var className;

    // Re-calculate and update displayed time
    function refresh () {
        var remain = element.remaining();
        var min = Math.floor(Math.abs(remain / 60000));
        var sec = (Math.floor(Math.abs(remain / 100)) / 10) % 60;

        // Set classes
        element.className = className;
        if ((! className) && (remain <= 20000)) {
            element.className += " lowtime";
        }
        if (! running) {
            element.className += " paused";
        }

        // Has the timer run out?
        if ((set_duration > 0) && (remain <= 0)) {
            duration = 0;
            sec = 0;
            running = false;
            if (callback) {
                callback();
            }
        }

        // .toFixed() rounds, we want to truncate
        if (! tenths) {
            sec = Math.floor(sec);
        } else {
            sec = sec.toFixed(1);
        }
        // Zero-pad
        if (sec < 10) {
            sec = "0" + sec;
        }

        var t = min + ":" + sec;
        if (t != element.innerHTML) {
            element.innerHTML = t;
        }
    }

    // Return remaining time in milliseconds
    element.remaining = function() {
        if (running) {
            var now = (new Date()).getTime();
            return duration - (now - startTime);
        } else {
            return duration;
        }
    }

    // Set timer to [d] milliseconds.
    // Put element into class [cn], if set.
    element.set = function(t, cn) {
        startTime = (new Date()).getTime();
        set_duration = t;
        duration = t;
        className = cn;
        refresh();
    }

    // Start timer
    element.start = function() {
        if (! running) {
            startTime = (new Date()).getTime();
            running = true;
        }
        refresh();
    }

    // Stop timer
    element.stop = function() {
        if (running) {
            duration = element.remaining();
            running = false;
        }
        refresh();
    }

    timer_updates.push(refresh);
}

// Transition state machine based on state
function transition(newstate) {
    var jt = e("jam");
    var pt = e("period");
    var jtext = e("jamtext");
    var jno = e("jamno");

    if ((newstate == undefined) || (newstate == state)) {
        return;
    }

    if ((state == SETUP) && window.penalties) {
        penalties_duck();
    }

    state = newstate;

    if (state == JAM) {
        pt.start();
        jt.set(120000);
        jt.start();
        jtext.innerHTML = jamtext[0];
        jamno += 1;
        jno.innerHTML = jamno;
    } else if (state == LINEUP) {
        pt.start();
        jt.set(30000, "lineup");
        jt.start();
        jtext.innerHTML = jamtext[1];
    } else if (state == TIMEOUT) {
        pt.stop();
        if (pt.remaining() <= 0) {
            pt.set(1800000);
        }
        jt.set(0, "timeout");
        jt.start();
        jtext.innerHTML = jamtext[2];
    }

    // Reset lead jammer indicators
    e("jammer-a").className = "";
    e("jammer-b").className = "";
}



/***********************************
 * Notices
 */

var notices = [
	false,
	'<embed src="res/Zounds.swf" type="text/html">',
	'<embed src="res/Ouch.swf" type="text/html">',
	'<embed src="res/Pow.swf" type="text/html">',
	'<embed src="res/HolyShot.swf" type="text/html">',
	'<embed src="res/FasterFaster.swf" type="text/html">',
	'<embed src="res/BadGirl.swf" type="text/html">',
	'<embed src="res/banana.gif" type="image/gif">',
];

var notice_timer;

function notice_expire() {
	var c = document.getElementById("notice");

	c.innerHTML = "";
}

function notice(n) {
    var c = document.getElementById("notice");

    if (notices[n]) {
        c.innerHTML = notices[n];
        clearTimeout(notice_timer);
        notice_timer = setTimeout(function() {notice_expire()}, 8000);
    } else {
        notice_expire();
    }
}

function e(id) {
    ret = document.getElementById(id);
    if (! ret) {
        return Array();
    }
    return ret;
}

function score(team, points) {
    var te = document.getElementById("score-" + team);
    var ts = Number(te.innerHTML);

    ts += points;
    te.innerHTML = ts;
}

/***********************************
 * Event handlers
 */

var logo = {a:-1, b:-1};

function leadJammer(team) {
    tgt = e("jammer-" + team);
    var on = ! tgt.className;

    e("jammer-a").className = "";
    e("jammer-b").className = "";
    if (on) tgt.className = "lead";
}

function handle(event) {
    var tgt = event.target || window.event.srcElement;
    var team = tgt.id.substr(tgt.id.length - 1);
    var adj = event.shiftKey?-1:1;
    var mod = (event.ctrlKey || event.altKey);
    var newstate;

    switch (tgt.id) {
    case "name-a":
    case "name-b":
        if (state == SETUP) {
            var tn = prompt("Enter team " + team + " name", tgt.innerHTML);

            if (tn) {
                tgt.innerHTML = tn;
            }
            if (window.penalties) {
                penalties_setTeamName(team, tn);
            }
        }
        break;
    case "logo-a":
    case "logo-b":
        if (state == SETUP) {
            if (mod) {
                var u = prompt("Enter URL to team " + team + " logo");

                if (u) {
                    tgt.src = u;
                }
            } else {
                var t, name;

                logo[team] = (teams.length + logo[team] + adj) % teams.length;
                t = teams[logo[team]];

                if (longnames) {
                    name = t[2];
                } else {
                    name = t[0];
                }

                e("name-" + team).innerHTML = name;
                tgt.src = "logos/" + t[1];

                if (window.penalties) {
                    penalties_setTeamName(team, t[0]);
                }
            }
        } else {
            score(team, -adj);
        }
        break;
    case "jammer-a":
    case "jammer-b":
        leadJammer(team);
        break;
    case "timeouts-a":
    case "timeouts-b":
        // Allow for timeouts > 3
        var v = Number(tgt.innerHTML);

        v -= adj;
        if (v == -1) v = 3;
        tgt.innerHTML = v;
        break;
    case "period":
        if ((state == SETUP) || (state == TIMEOUT)) {
            var r = prompt("Enter new time for period clock", tgt.innerHTML);
            if (! r) return;

            var t = r.split(":");
            var sec = 0;

            if (t.length > 3) {
                tgt.innerHTML = "What?";
                return;
            }

            for (var i in t) {
                var v = t[i];
                sec = (sec * 60) + Number(v);
            }

            tgt.set(sec*1000);
        } else {
            newstate = TIMEOUT;
        }
        break;
    case "periodtext":
        var pt;

        if (mod) {
            pt = prompt("Enter new period indicator text", tgt.innerHTML);
        } else {
            var ptl = periodtext.length;

            period = (period + ptl + adj) % ptl;
            pt = periodtext[period];
        }
        if (pt) {
            tgt.innerHTML = pt;
            if (state == TIMEOUT) {
                jamno = 0;
                e("jamno").innerHTML = jamno;
            }
        }
        break;
    case "jam":
        if (state == JAM) {
            newstate = LINEUP;
        } else {
            newstate = JAM;
        }
        break;
    case "jamno":
        jamno -= adj;
        tgt.innerHTML = jamno;
        break;
    case "score-a":
    case "score-b":
        if (state == SETUP) {
            var s = prompt("Enter score for team " + team, tgt.innerHTML);
            if (s) {
                tgt.innerHTML = s;
            }
        } else {
            score(team, adj);
        }
        break;
    }
    transition(newstate);
}

function key(event) {
    var e = event || window.event;
    var c = String.fromCharCode(e.which || e.keyCode || 0);
    var newstate;

    switch (c) {
    case " ":
        if (state == JAM) {
            newstate = LINEUP;
        } else {
            newstate = JAM;
        }
        break;
    case "t":
        newstate = TIMEOUT;
        break;
    case "a":
    case "[":
        score('a', 1);
        break;
    case "b":
    case "]":
        score('b', 1);
        break;
    case "A":
    case "{":
        score('a', -1);
        break;
    case "B":
    case "}":
        score('b', -1);
        break;
    case ",":
        leadJammer('a');
        break;
    case ".":
        leadJammer('b');
        break;
    case "1":
    case "2":
    case "3":
    case "4":
    case "5":
    case "6":
    case "7":
    case "8":
    case "9":
    case "0":
        var n = Number(c);

        window.notice(n);
    }

    transition(newstate);
}

function get(k, d) {
    if (! window.localStorage) {
        return d;
    } else {
        var v = window.localStorage["rdsb_" + k];
        if (v == undefined) {
            return d;
        }
        return v;
    }
}

function store(k, v) {
    if ((v == undefined) || ! window.localStorage) {
        return;
    } else {
        localStorage["rdsb_" + k] = v;
    }
}

function save() {
    if (window.penalties_save) {
        penalties_save();
    }
    store("period_clock", e("period").remaining());
    store("name_a", e("name-a").innerHTML);
    store("name_b", e("name-b").innerHTML);
    store("logo_a", e("logo-a").src);
    store("logo_b", e("logo-b").src);
    store("score_a", e("score-a").innerHTML);
    store("score_b", e("score-b").innerHTML);
    store("timeout_a", e("timeouts-a").innerHTML);
    store("timeout_b", e("timeouts-b").innerHTML);
    store("jamno", jamno);
    store("period", period);
}
    
function iecheck() {
    // If it's IE, it's got to be at least 7
    var ua = navigator.userAgent;
    var ie = ua.indexOf("MSIE ");

    if (ie == -1) {
        // Not IE
        return;
    } else {
        var n = parseFloat(ua.substring(ie + 5, ua.indexOf(";", ie)));
        if (n < 7) {
            alert("Your browser is too old to run the Woozle scoreboard.\nYou can use Firefox, Chrome, Opera, or Internet Explorer 7 and up.");
        }
    }
}

function start() {
    resize();
    iecheck();

    var p = document.getElementById("period");
    var j = document.getElementById("jam");
    var c;


    // IE8 doesn't have localStorage for file:// URLs  :<
    e("name-a").innerHTML = get("name_a", "Home");
    e("name-b").innerHTML = get("name_b", "Vis");
    e("logo-a").src = get("logo_a", "logos/black.png");
    e("logo-b").src = get("logo_b", "logos/white.png");
    e("score-a").innerHTML = get("score_a", 0);
    e("score-b").innerHTML = get("score_b", 0);
    e("timeouts-a").innerHTML = get("timeout_a", 3);
    e("timeouts-b").innerHTML = get("timeout_b", 3);
    period = Number(get("period", 0));
    jamno = Number(get("jamno", 0));

    if (window.localStorage) {
        save_itimer = setInterval(save, 1000);
    }
    
    if (window.penalties) {
        penalties_init();
    }

    e("periodtext").innerHTML = periodtext[period];
    e("jamtext").innerHTML = jamtext[3];
    transition();

    c = Number(get("period_clock", 1800000));
    startTimer(p);
    p.set(c);

    var j = document.getElementById("jam");
    startTimer(j, window.tenths);
    j.set(120000);

    save_timer = setInterval(save, 1000);
    update_itimer = setInterval(update, 33);

}

function resize() {
    var b = document.getElementsByTagName("body")[0];
    var w, h;
    
    // Internet Explorer makes everything a pain in the ass
    if (window.innerWidth) {
        w = window.innerWidth;
        h = window.innerHeight;
    } else if (document.documentElement && document.documentElement.clientWidth) {
        w = document.documentElement.clientWidth;
        h = document.documentElement.clientHeight;
    } else if (document.body) {
        w = document.body.clientWidth;
        h = document.body.clientHeight;
    } else {
        // Punt
        w = 800;
        h = 600;
    }
   
    w /= 7;
    h /= 5;

    var fs = Math.min(w, h);

    b.style.fontSize = fs + 'px';
}

window.onload = start;
document.onkeypress = key;  // IE requires document, not window
window.onresize = resize;
